@@ -27,6 +27,7 @@ import abc
 import netaddr
 import rfc3986
 import six
+from six.moves import range as compat_range
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -156,8 +157,8 @@ class String(ConfigType):
 
     def __repr__(self):
         details = []
-        if self.choices:
-            details.append("choices=%r" % self.choices)
+        if self.choices is not None:
+            details.append("choices={!r}".format(self.choices))
         if self.regex:
             details.append("regex=%r" % self.regex.pattern)
         if details:
@@ -296,7 +297,7 @@ class Number(ConfigType):
     def __repr__(self):
         props = []
         if self.choices is not None:
-            props.append("choices=%r" % (self.choices,))
+            props.append("choices={!r}".format(self.choices))
         else:
             if self.min is not None:
                 props.append('min=%g' % self.min)
@@ -439,23 +440,24 @@ class List(ConfigType):
         self.bounds = bounds
 
     def __call__(self, value):
-        if isinstance(value, list):
-            return value
+        if isinstance(value, (list, tuple)):
+            return list(value)
 
-        result = []
         s = value.strip()
-
         if self.bounds:
             if not s.startswith('['):
                 raise ValueError('Value should start with "["')
             if not s.endswith(']'):
                 raise ValueError('Value should end with "]"')
             s = s[1:-1]
+        if s:
+            values = s.split(',')
+        else:
+            values = []
+        if not values:
+            return []
 
-        if s == '':
-            return result
-
-        values = s.split(',')
+        result = []
         while values:
             value = values.pop(0)
             while True:
@@ -490,6 +492,62 @@ class List(ConfigType):
         return ','.join(value)
 
 
+class Range(ConfigType):
+
+    """Range type.
+
+    Represents a range of integers. A range is identified by an integer both
+    sides of a '-' character. Negatives are allowed. A single number is also a
+    valid range.
+
+    :param min: Optional check that lower bound is greater than or equal to
+                min.
+    :param max: Optional check that upper bound is less than or equal to max.
+    :param inclusive: True if the right bound is to be included in the range.
+    :param type_name: Type name to be used in the sample config file.
+
+    .. versionadded:: 3.16
+    """
+
+    def __init__(self, min=None, max=None, inclusive=True,
+                 type_name='range value'):
+        super(Range, self).__init__(type_name)
+        self.min = min
+        self.max = max
+        self.inclusive = inclusive
+
+    def __call__(self, value):
+        value = str(value)
+        num = "0|-?[1-9][0-9]*"
+        m = re.match("^(%s)(?:-(%s))?$" % (num, num), value)
+        if not m:
+            raise ValueError('Invalid Range: %s' % value)
+        left = int(m.group(1))
+        right = int(left if m.group(2) is None else m.group(2))
+
+        if left < right:
+            left = Integer(min=self.min)(left)
+            right = Integer(max=self.max)(right)
+            step = 1
+        else:
+            left = Integer(max=self.max)(left)
+            right = Integer(min=self.min)(right)
+            step = -1
+        if self.inclusive:
+            right += step
+        return compat_range(left, right, step)
+
+    def __eq__(self, other):
+        return (
+            (self.__class__ == other.__class__) and
+            (self.min == other.min) and
+            (self.max == other.max)
+        )
+
+    def _formatter(self, value):
+        return value
+
+
 class Dict(ConfigType):
 
     """Dictionary type.
@@ -700,19 +758,23 @@ class URI(ConfigType):
 
     Represents URI. Value will be validated as RFC 3986.
 
-    :param type_name: Type name to be used in the sample config file.
     :param max_length: Optional integer. If a positive value is specified,
                        a maximum length of an option value must be less than
                        or equal to this parameter. Otherwise no length check
                        will be done.
+    :param schemes: List of valid schemes.
+    :param type_name: Type name to be used in the sample config file.
 
     .. versionchanged:: 3.14
        Added *max_length* parameter.
+    .. versionchanged:: 3.18
+       Added *schemes* parameter.
     """
 
-    def __init__(self, max_length=None, type_name='uri value'):
+    def __init__(self, max_length=None, schemes=None, type_name='uri value'):
         super(URI, self).__init__(type_name=type_name)
         self.max_length = max_length
+        self.schemes = schemes
 
     def __call__(self, value):
         if not rfc3986.is_valid_uri(value, require_scheme=True,
@@ -722,6 +784,13 @@ class URI(ConfigType):
         if self.max_length is not None and len(value) > self.max_length:
             raise ValueError("Value '%s' exceeds maximum length %d" %
                              (value, self.max_length))
+
+        if self.schemes:
+            scheme = rfc3986.uri_reference(value).scheme
+            if scheme not in self.schemes:
+                raise ValueError("URI scheme '%s' not in %s" %
+                                 (scheme, self.schemes))
+
         self.value = value
         return value
 
